% a general form of numerical integral on reference domain
% 1D [0,1]

classdef pointMeasure 
    properties (SetAccess=protected, GetAccess=public)
        dim;
        N;
        p;
        w;
    end
    
    methods
        %constructor
        function A = pointMeasure(dim, q, w, p)
            if dim<=0 || q<=0 
                error('The dimension and number of points for integral approximation must be positive numbers');
            end
            if numel(w)~=q || ~isa(w,'numeric')
                error('The weights should be q numeric numbers');
            end
            if numel(p)~=(q*dim) || ~isa(p, 'numeric')
                error('The Abscissas should be dim*q numeric numbers');
            end
                
            A.dim = dim;
            A.N = q;
            A.w = reshape(w, q, 1);
            A.p = reshape(p, q, dim);
        end
        %
        function Q = mtimes(A, B)
            if ~isa(A,'pointMeasure') || ~isa(B,'pointMeasure')
                error('both entries of mtimes here should be in class pointMeasure');
            end
            dAB = A.dim+B.dim;
            NAB = A.N*B.N;
            pAB = [];
            wAB = [];
            for k = 1:B.N
                pAB = [pAB; A.p,repmat(B.p(k,:),A.N,1)];
                wAB = [wAB, A.w*B.w(k)];
            end 
            Q = pointMeasure(dAB, NAB, wAB, pAB);
        end
        %
        function I = integral(A,f)
            if ~isa(f, 'function_handle')
                error('A function handle is required');
            end
            I = A.w'*f(A.p);
        end
        %
        function plot(A)
            if A.dim == 1
                stem(A.p, A.w, 'filled');
            end
            if A.dim == 2
                stem3(A.p(:,1), A.p(:,2), reshape(A.w, A.N,1), 'filled');
            end
        end
            
    end
end