
classdef L2product < linearForm
    properties
        VS;  
        f ; 
        Q;
    end
    methods
        function  l2p = L2product(VS, f, Q)
            if ~isa(VS, 'solutionSpace')
                error('The 1st Argument is a function space');
            end
            if ~isa(f,'func') || f.domain~=VS.domain || f.range~=VS.range
                error('The 2nd Argument is a function with the same domain and range as the function space');
            end
            if ~isa(Q,'Measure.pointMeasure')
                error('The 3rd Argument is a point measure');
            end
            
            % the values of the reference basis at the points for point
            % measure
            bsVal = VS.dualbs*VS.vsRef.Bs.id.at(Q.p);
            
            % fv is a row vector 
            fv = zeros(VS.dim,1); 
            % calculate the components of fv
            tr = VS.domain;
            for k = 1:VS.dim
               dom = VS.compDom{k};
               for c = 1:size(dom, 1)
                 xp = tr.get_sInCell(dom(c,1), Q.p); 
                 fv(k) = fv(k) + bsVal(dom(c,2),:)*(Q.w.*f.at(xp))*det(tr.cell2Aff{dom(c,1)}{1});
               end
            end
            l2p@linearForm(VS.Bs, vector(VS.Bs,fv));
            l2p.VS = VS;
            l2p.f = f;
            l2p.Q = Q;
        end
    end
end