% a general form of bilinear

classdef bilinearForm
    properties(SetAccess=private)
      M; 
      Bs;
    end
    methods
        function A = bilinearForm(bs, M)
            if ~isa(bs,'basis') || size(M,1)~=bs.dim || size(M,2)~=bs.dim
                error('The bilinear matrix form must coincides with the basis.')
            end
            A.M = sparse(M);
            A.Bs = bs;
        end
        %
        function z = apply(A, u,v)
            if ~isa(u,'vector') || ~isa(v,'vector') || u.Bs~=v.Bs || u.Bs~=A.Bs
                error('The bilinear works on the vectors in different Vector space');
            end
            K=A.Bs.dim;
            z=reshape(v.c, 1, K)*full(A.M)*reshape(u.c, K,1);
        end
        %
        function z = mldivide(A, b) 
            if (~isa(b,'vector') && ~isa(b,'linearForm')) || b.Bs~=A.Bs 
                error('The linear system is not given properly.');
            end
            if isa(b,'vector')
              z = vector(A.bs, full(A.M)\b.c);
            end
            if isa(b,'linearForm')
               z = vector(A.Bs, full(A.M)\reshape(b.f, A.Bs.dim,1));
            end
        end
    end
end