

classdef func
    properties
        domain;
        range;
        map;
        d;
    end
    methods
        function fc = func(dm, rg, g, varargin)
            if ~isa(dm,'triangulation')
                error('The 1st argument is the domain (type: triangulation) of a function');
            end
            if ~isa(rg,'field')
                error('The 2nd argument is the range (type: field) of a function');
            end
            if ~isa(g,'function_handle')
                error('the 3rd argument is a function handle');
            end
            fc.domain = dm;
            fc.range = rg;
            fc.map = g;
            fc.d = cell(dm.dim,1);
            if ~isempty(varargin)
                for k=1:dm.dim
                 fc.d{k} = func(dm,rg,varargin{1}{k});
                end
            end
        end
        
        % evaluate the function value at point x
        function s = at(fc,x)
            if ~isa(fc,'func') || ~isa(x,'numeric')
                error('The arguments must be a function and evalution points')
            end
            
            if size(x,2)==fc.domain.dim
                s = fc.map(x); 
            elseif size(x,1)==fc.domain.dim
                s = fc.map(x');
                s = s';
            else
                error('x is not in the domain of the function');
            end                
        end
        %
        function s = plus(fc,f)
            if ~isa(fc,'func') || ~isa(f,'func') || fc.domain~=f.domain || f.range~=fc.range
                error('The arguments are not functions or functions with differenct definition');
            end
            s = func(fc.domain, fc.range, @(x) fc.at(x) + f.at(x));
            s.d = cell(s.domain.dim,1);
            for k=1:length(fc.d)
                if ~isempty(fc.d{k}) && ~isempty(f.d{k})
                  s.d{k} = func(s.domain, s.range, @(x)fc.d{k}.at(x) + f.d{k}.at(x));
                end
            end
        end
        %
        function s=mtimes(fc,a)
            if isa(a,'func') && isa(fc,'numeric') && numel(fc)==1
                s = func(a.domain, a.range, @(x) fc*a.at(x));
                s.d = cell(s.domain.dim,1);
                for k=1:length(a.d) 
                 if ~isempty(a.d{k})
                  s.d{k} = func(s.domain, s.range, @(x) fc*a.d{k}.at(x)); 
                 end
                end
            end
            if isa(fc,'func') && isa(a,'numeric') && numel(a)==1
                s = func(fc.domain,fc.range, @(x) a*fc.at(x));
                s.d = cell(s.domain.dim,1);
                for k=1:length(fc.d)
                 if ~isempty(fc.d{k})
                  s.d{k} = func(s.domain, s.range, @(x) a*fc.d{k}.at(x));
                 end
                end
            end
            if isa(a,'func') && isa(fc,'func') && a.domain==fc.domain && a.range==fc.range
                s = func(fc.domain, fc.range, @(x) a.at(x).*fc.at(x));
                s.d = cell(s.domain.dim,1);
                for k=1:length(fc.d)
                 if ~isempty(fc.d{k}) && ~isempty(a.d{k})
                  s.d{k} = func(s.domain, s.range, @(x) fc.d{k}.at(x).*a.at(x)+fc.at(x).*a.d{k}.at(x));
                 end
                end
            end                
        end

        % get the derivative in direction x_k
        function dr = derivative(fc,k)
            if ~isa(fc, 'func') || ~isa(k, 'numeric')
                error('The argument must be a function and positive integer numbers');
            end
            if length(k)>1 || ~isequal(k,abs(floor(k))) || k>fc.domain.dim
                error('This is to single out the k-th partial derivative'); 
            end
            
            dr = fc.d{k};
        end
        %
        function plot_func(fc)
            if fc.domain.dim==1
               figure  
                xmin = min(fc.domain.nodes);
                xmax = max(fc.domain.nodes);
                nx = 150;
                x=rand(nx,fc.domain.dim)*diag(xmax)+repmat(xmin, nx, 1);
                y = fc.at(x);
                plot(x', y','.')  
            else
                display('there is no display for systems in 2D and 3D');
            end        
        end
        % test the derivative function
        function df = testDerivative(fc,k)           
            if ~isa(fc, 'func') || ~isa(k, 'numeric') 
                error('The first argument is a function and the second is the direction of the derivative');
            end
            dim = fc.domain.dim;
            if  k<=0 || k>dim
                error('The second argument is the dimensional index');
            end
            dr = derivative(fc, k);
            h=1./[10,20,30,40,50];
            err = zeros(1,length(h));
            x = fc.domain.nodes;
            dr.at(x)
            for hk = 1:length(h)
                dx = repmat([zeros(1,k-1),h(hk),zeros(1,dim-k)], size(x,1),1);
                err(k) = max(abs(fc.at(x+dx)+fc.at(x-dx)-2*fc.at(x)-dr.at(x)));
            end
            p=polyfit(log(h),log(err), 1);
            df = 0;
            if p(1)>1.5
                df = 1;
            end
        end
    end
end