
classdef laplace < bilinearForm
    properties
        Q;
        V;
    end
    methods
        function lap = laplace(VS,Q)
            if ~isa(VS,'vectorSpace')
                error('1st argument is a function space');
            end
            if ~isa(Q,'Measure.pointMeasure')
                error('2nd argument is a point measure');
            end
            
            % The derivate's value of the basis on the reference at measure
            % points are calculated 
            % only for 1D now
            Mp = zeros(VS.vsRef.Bs.dim, numel(Q.w));
            for k = 1:VS.vsRef.Bs.dim
              Mp(k,:)=reshape(VS.vsRef.Bs.id.funcs{k}.d{1}.at(Q.p), 1, numel(Q.w));
            end
            derivVal = VS.dualbs*Mp;
              
            
            % the domain
            tr = VS.domain; 
            
            
                
            row=[];
            col=[];
            val=[];
            % diagonal elements of the laplace 
            for i=1:VS.dim
                icell = VS.compDom{i};
                row = [row; i];
                col = [col; i];
                ev = 0;
                for k=1:size(icell,1)  
                  Aff = tr.cell2Aff{icell(k,1)}{1}; 
                  ev = ev + (derivVal(icell(k,2),:).^2*Q.w)*det(Aff)/Aff/Aff;
                end
                val = [val; ev];
            end  
             %
               for i = 1:VS.dim
                   for j = (i+1):VS.dim   % laplace is symmetric
                       icell = VS.compDom{i};
                       jcell = VS.compDom{j};
                       [commCell, ik,jk] = intersect(icell(:,1),jcell(:,1));
                       if ~isempty(commCell)
                         row = [row; i; j];% laplace is symmetric
                         col = [col; j; i]; 
                         Aff = zeros(1,numel(commCell));              
                         for k=1:numel(commCell)
                             Aff(k) = tr.cell2Aff{commCell(k)}{1};
                         end
                         ev = 1./Aff *(derivVal(icell(ik,2),:).*derivVal(jcell(jk,2),:))*Q.w; 
                         val = [val; ev; ev];
                       end
                   end
               end  
             lap@bilinearForm(VS.Bs, sparse(row, col, val, VS.dim, VS.dim)); 
             lap.Q = Q;
             lap.V = VS;
            end
    end 
end