
classdef ptBasis < indepFuncSys
    properties
        t ;      % degree
        dimP;    % dimension of this function system
        inDim;   % the number of nodes which are not the cell nodes
        indepNodes = []; % coordinaten of the points which the solution will be calculated
        compactDom ; % each function is not zero on a small compact domain
    end
    methods
        function pt = ptBasis(dm, rg, t)
            if ~isa(dm, 'triangulation')
                error('The domain is a triangulation');
            end
            if ~isa(rg,'field')
                error('The function range is a field');
            end
            if ~isinteger(int8(t)) || length(t)~=1
                error('The degree of the monomials must be a integer');
            end
            
            % pt functions on Reference (now only for 1D) 
                % now the reference is a segment
                Tref = dm.ref{dm.cell2ref};  
                monomils = Monombasis(Tref, rg, t);
                vsRef = vectorSpace(Tref, rg, monomils);                
                Id = eye(vsRef.dim);
                dualbs = get_dualbasis(vsRef, Id);  
                %
                g = cell(vsRef.dim,1);
                dg = cell(vsRef.dim,1);  
                g{1} = vector2func(vsRef, vector(vsRef.Bs, dualbs(1,:)));
                dg{1} = g{1}.d;  
                g{2} = vector2func(vsRef, vector(vsRef.Bs, dualbs(vsRef.dim,:)));
                dg{2} = g{2}.d;
                for k=3:vsRef.dim
                  g{k} = vector2func(vsRef, vector(vsRef.Bs, dualbs(k-1,:)));
                  dg{k} = g{k}.d;
                end 
            
             %  
                sP = vsRef.Bs.id.p(2:end-1);
                inP = length(sP); 
                dimp = dm.ncell*inP+dm.nnode;
                phi = cell(dimp,1);
                compDom = cell(dimp,1);
                for k=1:dm.nnode
                  compDom{k} = dm.get_node2cell(k); 
                  phi{k} = func(dm,rg,@(x) ptBasis.node_phi(g, dm, compDom{k}, x)); 
                  for d = 1:dm.dim
                    phi{k}.d{d} = func(dm,rg,@(x) ptBasis.node_phi(dg, dm, compDom{k}, x, d));
                  end 
                end
                for k=1:inP
                    for c = 1:dm.ncell
                        ik = dm.nnode+(c-1)*inP+k; 
                        compDom{ik} = [c, size(dm.cell2node,2)+k];
                        phi{ik} = func(dm,rg, @(x) ptBasis.node_phi(g,dm,compDom{ik},x));  
                        for d = 1:dm.dim 
                          phi{ik}.d{d} = func(dm,rg,@(x) ptBasis.node_phi(dg, dm, compDom{ik}, x,d));
                        end 
                    end
                end
               % 
               inNode = zeros(dm.ncell*inP, dm.dim);
               for k = 1:inP
                 for c = 1:dm.ncell
                     inNode((c-1)*inP+k,:) = dm.get_sInCell(c, sP(k));
                 end
               end
               % independence test points
               testP = [dm.nodes; inNode];
               Vs = vectorSpace(dm, rg);
               Lambda = cell(dimp,1);
               for k = 1:dimp
                 Lambda{k} = delta(testP(k), Vs);
               end
               pt@indepFuncSys(phi, Lambda);
               if det(pt.vdM)
                   display('here is a available basis');
               end
               pt.t = t;
               pt.dimP = dimp;
               pt.inDim = inP;
               pt.indepNodes = testP;
               pt.compactDom = compDom;
        end
    end
    %private methods
    methods (Static, Access=private)
        function val = node_phi(g, dm, kdomain, x, varargin)
            % evaluate the k-th function at x
            %          or the function in corresponding the vargarin-th point in the cell k at x.
            if ~isa(dm,'triangulation')
                error('The function domain is a triangulation');
            end
            if size(kdomain,2)~=2
                error('The second argument is the compact domain');
            end
            if size(x,2)==dm.dim
                val = zeros(size(x,1),1);
            end
            if size(x,1)==dm.dim
                val = zeros(1,size(x,2)); 
            end
            % check if x in the compact domain of the function
            % yes, xin = 1, and return the cell number in nT and the corresponding position sx in the
            %      reference cell
            % no,  xin = 0 
            [xin, nT,sx]=check_xInDomain(dm, x, kdomain(:,1));  
            for k = 1:numel(xin) 
             if xin(k)  
                i0 = kdomain(kdomain(:,1)==nT(k),2);
                if isempty(varargin)
                   val(k) = g{i0}.at(sx(k));  
                else 
                   val(k) = g{i0}{varargin{1}}.at(sx(k))/det(dm.cell2Aff{nT(k)}{1}); 
                end
             end
            end
        end
        %
    end
end