
classdef ptBasis < indepFuncSys
    properties
        t ;      % degree
        dimP;    % dimension of this function system
        inDim;   % the number of nodes which are not the cell nodes
        indepNodes = []; % coordinaten of the points which the solution will be calculated
        compactDom ; % each function is not zero on a small compact domain
    end
    methods
        function pt = ptBasis(dm, rg, t)
            if ~isa(dm, 'triangulation')
                error('The domain is a triangulation');
            end
            if ~isa(rg,'field')
                error('The function range is a field');
            end
            if ~isinteger(int8(t)) || length(t)~=1
                error('The degree of the monomials must be a integer');
            end
            
            % pt functions on Reference (now only for 1D)    
                g = cell(t+1,1);
                dg = {};
                if isempty(dm.cell2ref)
                  % domain is a reference cell 
                   error('The basis is given ....');
                else    
                   Tref = dm.ref{dm.cell2ref};  
                end
                if dm.dim == 1 && t==1
                    g{1} = func(Tref,rg,@(x) 1-x); 
                    g{2} = func(Tref,rg,@(x) x); 
                    % derivatives 
                    dg{1} = func(Tref,rg,@(x) -1);
                    dg{2} = func(Tref,rg,@(x) 1);                    
                    inP = 0;
                end
                if dm.dim ==1 && t==2
                    % at nodes
                    g{1} = func(Tref,rg, @(x) (2*x-1).*(x-1));
                    g{2} = func(Tref,rg, @(x) x.*(2*x-1)); 
                    dg{1} = func(Tref,rg, @(x) 4*x-3);
                    dg{2} = func(Tref,rg, @(x) 4*x-1);
                    % at inner point in the cell
                     g{3} = func(Tref,rg, @(x) 4*x.*(1-x));
                     dg{3} = func(Tref,rg, @(x) 4-8*x);
                     sP = 0.5;
                     inP=1;
                end 
             %   
                dimp = dm.nnode + dm.ncell*inP;
                phi = cell(dimp,1);
                compDom = cell(dimp,1);
                for k=1:dm.nnode
                  compDom{k} = dm.get_node2cell(k); 
                  phi{k} = func(dm,rg,@(x) ptBasis.node_phi(g, dm, compDom{k}, x));
                  if ~isempty(dg)
                      dphi=1;
                    for d = 1:dm.dim
                      phi{k}.d{d} = func(dm,rg,@(x) ptBasis.node_phi(dg, dm, compDom{k}, x, dphi));
                    end
                  end
                end
                for k=1:inP
                    for c = 1:dm.ncell
                        ik = dm.nnode+(c-1)*inP+k; 
                        compDom{ik} = [c, size(dm.cell2node,2)+k];
                        phi{ik} = func(dm,rg, @(x) ptBasis.node_phi(g,dm,compDom{ik},x)); 
                        if ~isempty(dg) 
                         for d = 1:dm.dim
                          phi{ik}.d{d} = func(dm,rg,@(x) ptBasis.node_phi(dg, dm, compDom{ik}, x,dphi));
                         end
                        end
                    end
                end
               % 
               inNode = zeros(dm.ncell*inP, dm.dim);
               for k = 1:inP
                 for c = 1:dm.ncell
                     inNode((c-1)*inP+k,:) = dm.get_sInCell(c, sP);
                 end
               end
               % independence test points
               testP = [dm.nodes; inNode];
               pt@indepFuncSys(dimp, phi, testP);
               if det(pt.at(testP))
                   display('here is a available basis');
               end
               pt.t = t;
               pt.dimP = dimp;
               pt.inDim = inP;
               pt.indepNodes = testP;
               pt.compactDom = compDom;
        end
    end
    %private methods
    methods (Static, Access=private)
        function val = node_phi(g, dm, kdomain, x, varargin)
            % evaluate the k-th function at x
            %          or the function in corresponding the vargarin-th point in the cell k at x.
            if ~isa(dm,'triangulation')
                error('The function domain is a triangulation');
            end
            if size(kdomain,2)~=2
                error('The second argument is the compact domain');
            end
            if size(x,2)==dm.dim
                val = zeros(size(x,1),1);
            end
            if size(x,1)==dm.dim
                val = zeros(1,size(x,2)); 
            end
            % check if x in the compact domain of the function
            % yes, xin = 1, and return the cell number in nT and the corresponding position sx in the
            %      reference cell
            % no,  xin = 0 
            [xin, nT,sx]=check_xInDomain(dm, x, kdomain(:,1));  
            for k = 1:numel(xin) 
             if xin(k)  
                i0 = kdomain(kdomain(:,1)==nT(k),2);
                val(k) = g{i0}.at(sx(k));  
                if ~isempty(varargin)
                  val(k) = val(k)/det(dm.cell2Aff{nT(k)}{1}); 
                end
             end
            end
        end
        %
    end
end