% finite element method = triple(Th, Pi, Sum)
% Th : triangulation of domain
% Pi : function space on the reference
%Sum : linear independent functionals for Pi
% note: Pi and Sum appear together as a pair.

classdef solutionSpace < vectorSpace 
    properties
        t ;      % degree 
        vsRef;   % function space on the reference
        dualbs;  % dual basis with lagrange polynomials
        xnode;   % freedom points
        compDom; % compact domains for the freedom nodes.
    end
    methods
        function pt = solutionSpace(dm, rg, t)
            if ~isa(dm, 'triangulation')
                error('The domain is a triangulation');
            end
            if ~isa(rg,'field')
                error('The function range is a field');
            end
            if ~isinteger(int8(t)) || length(t)~=1
                error('The degree of the monomials must be a integer');
            end
            
            % Pi (now only for 1D, the reference is a segment) 
            pt@vectorSpace(dm,rg);
            Tref = dm.ref{dm.cell2ref};  
            monomils = Monombasis(Tref, rg, t);
            pt.vsRef = vectorSpace(Tref, rg, monomils);   
            pt.dualbs = get_dualbasis(pt.vsRef, eye(pt.vsRef.dim));  
            pt.t = t;   
            [pt.xnode, pt.compDom]= get_freedomPoint(pt);
            pt.dim = size(pt.xnode, 1);
            % we do not need to get the concrete basis of pt, which are
            % constructed from the basis on the reference cell.
            pt.Bs = basis(funcSys(pt.dim), pt.dim);
        end
        %
        function [xp, compDom] = get_freedomPoint(pt) 
        % freedom points and their compact domains (if wanted)
           dm = pt.domain;
           inP = pt.t-1;
           inNode = zeros(dm.ncell*inP, dm.dim);
           compDom = cell(dm.nnode+dm.ncell*inP,1);
           for k=1:dm.nnode
                  compDom{k} = dm.get_node2cell(k);
           end
           for c = 1:dm.ncell
              for k = 1:inP
               ik = (c-1)*inP+k; 
               compDom{dm.nnode+ik} = [c, size(dm.cell2node,2)+k];
               inNode(ik,:) = dm.get_sInCell(c, pt.vsRef.Bs.id.p(k+1));
              end
           end
           % independence test points
           xp = [dm.nodes; inNode];
        end
    end
end