
% we want to solve the problem -laplace(u)=f with f=sin(x) on [0,2pi]
%
% the exact solution: u=sin(x)


close all
clear classes

dom = [0,2*pi];
bctyp={'Dirichlet', 'Neumann'};
bcpos={[0,2*pi], []};  % two ends of the dom are Dirichlet boundary nodes.
if ismember('Dirichlet', bctyp)
    dirich = [0,0];  % boundary value
end
%right side function
fright = @(x) sin(x);

% triangulation
N=11;
tr = tri1D(dom,N, bctyp, bcpos);
% right side
R = field('R');
f = func(tr, R, fright);

% choose shape function and build up the function space
% p_1 element
V = solutionSpace(tr, R, 1);
% choose point measure
Q = Measure.gaussLegend(2);
% linear form
fv = L2product(V,f,Q);
% bilinear form
auv = laplace(V,Q);

%consider boundary condition
if ~isempty(tr.Dirichlet)
    bcnode = tr.Dirichlet; % the value of the solution at Dirichlet points are known
    dirich = reshape(dirich, numel(bcnode),1);
    unknown = setdiff(1:V.dim, bcnode);
    A = auv.M(unknown, unknown);
    b = fv.b(unknown)'- full(auv.M(unknown, bcnode))*dirich;
end

% solution usol at point x
    usol = [A\b; dirich]'; 
    xind = [unknown, bcnode];
    
% in one D, one can sort the point orderly
    [x, I] = sort(V.xnode(xind));
    usol = usol(I);
    
    figure
    plot(x', usol,'o')
    
    
    