%no help for you
classdef vector
    properties(SetAccess=protected)
      %basis
      Bs
      %coordinate vector
      c
    end
    
    methods
        % constructor
        function v = vector(bas,varargin)
            if ~isa(bas,'basis')
                error('First argument must be a basis.');
            end
            v.Bs = bas;
            if isempty(varargin)
                v.c = zeros(v.Bs.dim, 1);
            else
                coo = varargin{1};
                if ~isnumeric(coo)
                  error('Coordinates (2nd argument) must be of numeric type.');
                end
                if numel(coo)~=bas.dim
                  error('Coordinate vector has wrong dimension.');
                end
                v.c = reshape(coo,v.Bs.dim,1);
            end
        end
        
        % oveloading +
        function z = plus(this,x)
            vector.check(this,x); 
            z = vector(this.Bs,this.c + x.c);
        end
        % oveloading -
        function z = minus(this,x)
            vector.check(this,x);
            z = vector(this.Bs,this.c - x.c);
        end
        
        % oveloading -u
        function z = uminus(this) 
            z = vector(this.Bs, - this.c);
        end        
        % oveloading *
        function z = mtimes(this,x) 
            temp = isa(x,'vector')*2 + isa(this,'vector');
            switch temp
                case 3                  
                  if  this.Bs~=x.Bs    
                      error('Multiplication not defined.');
                  end 
                   z = this.c' * x.c; 
                case 2
                    if ~isa(this,'numeric') && length(this)~=1  
                        error('Multiplication not defined.');
                    end
                        z = vector(x.Bs, this*x.c); 
                case 1
                    if ~isa(x,'numeric') && length(x)~=1
                      error('Multiplication not defined.');
                    end
                        z = vector(this.Bs, x*this.c); 
                case 0
                     error('Multiplication not defined.');
            end
         end      
        
        % oveloading ==
        function z = eq(this,x)
            vector.check(this,x); 
            z = sum(this.c==x.c)==length(x.c);
        end 
        
        % oveloading ~=
        function z = ne(this,x) 
            vector.check(this,x);
            z = sum(this.c==x.c)~=length(x.c);
        end   
 
    end
    
    methods(Static,Access=private)
        function check(this,x)
            if ~isa(x,'vector') || ~isa(this,'vector') || this.Bs~=x.Bs
                error('Operation not defined.');
            end 
        end
    end
end
